@@ -14,22 +14,20 @@ from .utils import (
     Flask,
     LocalStorage,
     RedisStorage,
-    MongoStorage,
     JsonResponse,
 )
 from ._installer import PluginInstaller
 from ._web import blueprint
 
-__author__ = "staugur <me@tcw.im>"
+__author__ = "Hiroshi.tao <me@tcw.im>"
 
-__version__ = "3.6.2"
+__version__ = "3.7.1"
 
 __all__ = [
     "Flask",
     "PluginManager",
     "LocalStorage",
     "RedisStorage",
-    "MongoStorage",
     "JsonResponse",
     "PluginInstaller",
     "push_dcp",
